#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define max(a,b) a>b?a:b
#define Nobs     100             //length of the series
#define filename "hjnote.out"    //output file



#define CV       1.6448530004709 //5% one-sized crit value
#define SEED  -1                 //random seed

//ARCH parameters
#define alpha    1
#define beta     0.4


int NN=1000;                     // number of simulations
int m=1;                         // embedding dimension
int n;
double Q, **A, C[4];


/* determine the log of ratios of correlation integrals */

void redun(double *x, double *y, int N, int m, int mmax, double epsilon)
{

  int i, j, s;
  double disx, disy, disz, Cy, Cxy, Cyz, Cxyz;

  Q=Cy=Cxy=Cyz=Cxyz=0.0;
  n = N - mmax;

  for (i=mmax;i!=N;i++)
  {
    for (j=mmax;j!=N;j++)
    if (j!=i)
    {
      disx = disy = 0.0;
      for (s=1;s!=m+1;s++)
        disx = max(fabs(x[i-s]-x[j-s]),disx);

      for (s=1;s!=mmax+1;s++)
        disy = max(fabs(y[i-s]-y[j-s]),disy);

      if (disy <= epsilon)
      {
	Cy++;
        A[3][i]++;

        if (disx <= epsilon)
        {
          Cxy++;
          A[1][i]++;
        }

        disz = max(fabs(y[i]-y[j]),disy);
        if (disz <= epsilon)        {
          Cyz++;
          A[2][i]++;
          if (disx <= epsilon)
          {
            Cxyz++;
            A[0][i]++;
          }
        }
      }   // end condition |Yi - Yj| < epsilon
    }   // end loop over j
  } // end loop over i

  Q = (double) Cxyz/Cxy - (double) Cyz/Cy;

  C[0] = Cxyz/(double)(n*(n-1));
  C[1] = Cxy/(double)(n*(n-1));
  C[2] = Cyz/(double)(n*(n-1));
  C[3] = Cy/(double)(n*(n-1));

  for (i=0;i!=4;i++)
   for (j=mmax;j!=N;j++)
   {
     A[i][j] /= (double)(n-1);
     A[i][j] -= C[i];
   }
}

/* normalise the time series to unit std. dev. */

void normalise(double *x, int N)
{
  int i;
  double mean=0.0, var=0.0;

  for (i=0;i!=N;i++)
  {
    mean += x[i];
    var += x[i]*x[i];
  }

  mean /= (double)(N);
  var /= (double)(N);
  var -= mean*mean;

  for (i=0;i!=N;i++)
    x[i] = (x[i]-mean)/sqrt(var);

  return;
}

// simulates Normal (0,1)
double gasdev(void)
{
	static int iset=0;
	static double gset;
	double fac,rsq,v1,v2;

	if  (iset == 0) {
		do {
			v1=2.0*((double)(rand())/(double)(RAND_MAX))-1.0;
			v2=2.0*((double)(rand())/(double)(RAND_MAX))-1.0;
			rsq=v1*v1+v2*v2;
		} while (rsq >= 1.0 || rsq == 0.0);
		fac=sqrt(-2.0*log(rsq)/rsq);
		gset=v1*fac;
		iset=1;
		return v2*fac;
	} else {
		iset=0;
		return gset;
	}
}

int main()
{

  double x[Nobs], y[Nobs], *ohm, S2, mu,\
  HJ_TVAL, HJ_mean, HJ_sigma, d[4], sigma[4][4], hv, size;
  int i, ieps, j, l, k, K, nn, mmax;
  FILE *fil;

  double epsilon=1.0;  // set the bandwidth


  A = (double **) malloc(4*sizeof(double *));

  for (i=0;i!=4;i++)
   A[i] = (double *) malloc(Nobs*sizeof(double));

  K = (int)(sqrt(sqrt(Nobs-m)));

  ohm = (double *) malloc(K*sizeof(double));

  ohm[0] = 1.0;
  for (k=1;k<K;k++)
    ohm[k] = 2.0*(1.0-k/(double)(K));

  srand(SEED); //set random seed or use randomize();

  HJ_mean=HJ_sigma=size=0.0;

  for (nn=0;NN!=nn;nn++)   //iteration loop
  {
   printf("Iteration: %i      \r", nn);
  //Generate series

   hv=sqrt(alpha/(1-beta));

  //    transient period
    for (i=1;i<100;i++)
    {
       x[0]=gasdev()*sqrt(hv);
       hv=alpha+beta*x[0]*x[0];
       y[0]=gasdev()*sqrt(hv);
    }
    for (i=0;i<Nobs;i++)
    {
       x[i]=gasdev()*sqrt(hv);
       hv=alpha+beta*x[i]*x[i];
       y[i]=gasdev()*sqrt(hv);
    }

    for (j=0;j!=4;j++)
    {
      C[j] = 0.0;
      for (i=0;i!=Nobs;i++)
        A[j][i] = 0.0;
    }


    normalise(x, Nobs);
    normalise(y, Nobs);

    mmax=m;

    redun(y,x,Nobs,m,mmax,epsilon);

    for (i=0;i!=4;i++)
      for (j=0;j!=4;j++)
      {
        sigma[i][j] = 0.0;
        for (k=0;k!=K;k++)
          for (l=mmax+k;l!=Nobs;l++)
            sigma[i][j] += 4.0*ohm[k]*(A[i][l]*A[j][l-k]+A[i][l-k]*A[j][l])/(double)(2*(n-k));
      }

    d[0] = 1.0/C[1];
    d[1] = -C[0]/(C[1]*C[1]);
    d[2] = -1.0/C[3];
    d[3] = C[2]/(C[3]*C[3]);

    S2=0.0;
    for (i=0;i!=4;i++)
      for (j=0;j!=4;j++)
         S2 += d[i]*sigma[i][j]*d[j];

    HJ_TVAL = Q*sqrt(n)/sqrt(S2);

    if (HJ_TVAL>=CV)
       size++;

    HJ_mean += HJ_TVAL;
    HJ_sigma += HJ_TVAL*HJ_TVAL;
  }// end of iteration loop

  HJ_mean /= (double) NN;
  HJ_sigma /= (double) NN;
  HJ_sigma = sqrt(HJ_sigma - HJ_mean*HJ_mean);
  size /= (double) NN;

  fil=fopen(filename,"w");
  fprintf(fil, "SampleN, Epsilon, HJsize, HJmean, HJsigma\n");
  fprintf(fil, "%i,%f,%f,%f,%f\n",Nobs, epsilon, size, HJ_mean, HJ_sigma);
  fclose(fil);

  return(0);
}

