#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define max(a,b) a>b?a:b
#define Nobs     100             //length of the series
#define infile1   "data1"        //input file1, causing
#define infile2   "data2"        //input file2, being caused 
#define outfile  "hjstat.out"    //output file

double epsilon=1.0;              // set the bandwidth
int m=1;                         // embedding dimension
int n;
double Q, **A, C[4];


/* determine the log of ratios of correlation integrals */

void redun(double *x, double *y, int N, int m, int mmax, double epsilon)
{

  int i, j, s;
  double disx, disy, disz, Cy, Cxy, Cyz, Cxyz;

  Q=Cy=Cxy=Cyz=Cxyz=0.0;
  n = N - mmax;

  for (i=mmax;i!=N;i++)
  {
    for (j=mmax;j!=N;j++)
    if (j!=i)
    {
      disx = disy = 0.0;
      for (s=1;s!=m+1;s++)
        disx = max(fabs(x[i-s]-x[j-s]),disx);

      for (s=1;s!=mmax+1;s++)
        disy = max(fabs(y[i-s]-y[j-s]),disy);

      if (disy <= epsilon)
      {
	Cy++;
        A[3][i]++;

        if (disx <= epsilon)
        {
          Cxy++;
          A[1][i]++;
        }

        disz = max(fabs(y[i]-y[j]),disy);
        if (disz <= epsilon)
        {
          Cyz++;
	  A[2][i]++;
          if (disx <= epsilon)
          {
            Cxyz++;
            A[0][i]++;
          }
        }
      }   // end condition |Yi - Yj| < epsilon
    }   // end loop over j
  } // end loop over i

  Q = (double) Cxyz/Cxy - (double) Cyz/Cy;

  C[0] = Cxyz/(double)(n*(n-1));
  C[1] = Cxy/(double)(n*(n-1));
  C[2] = Cyz/(double)(n*(n-1));
  C[3] = Cy/(double)(n*(n-1));

  for (i=0;i!=4;i++)
   for (j=mmax;j!=N;j++)
   {
     A[i][j] /= (double)(n-1);
     A[i][j] -= C[i];
   }
}

/* normalise the time series to unit std. dev. */

void normalise(double *x, int N)
{
  int i;
  double mean=0.0, var=0.0;

  for (i=0;i!=N;i++)
  {
    mean += x[i];
    var += x[i]*x[i];
  }

  mean /= (double)(N);
  var /= (double)(N);
  var -= mean*mean;

  for (i=0;i!=N;i++)
    x[i] = (x[i]-mean)/sqrt(var);

  return;
}

int main()
{
  double x[Nobs], y[Nobs], *ohm, S2, \
    HJ_TVAL, HJ_Pval, d[4], sigma[4][4], hv;
  int i, ieps, j, l, k, K, nn, mmax;
  FILE *fil;



  A = (double **) malloc(4*sizeof(double *));

  for (i=0;i!=4;i++)
   A[i] = (double *) malloc(Nobs*sizeof(double));

  K = (int)(sqrt(sqrt(Nobs-m)));

  ohm = (double *) malloc(K*sizeof(double));

  ohm[0] = 1.0;
  for (k=1;k<K;k++)
    ohm[k] = 2.0*(1.0-k/(double)(K));

//get external data 

    fil=fopen(infile1,"r");
    for (i=0;i<Nobs;i++)
       fscanf(fil,"%lf",&x[i]);
    fclose(fil);

    fil=fopen(infile2,"r");
    for (i=0;i<Nobs;i++)
       fscanf(fil,"%lf",&y[i]);
    fclose(fil);

    for (j=0;j!=4;j++)
    {
      C[j] = 0.0;
      for (i=0;i!=Nobs;i++)
        A[j][i] = 0.0;
    }


    normalise(x, Nobs);
    normalise(y, Nobs);

    mmax=m;

    redun(x,y,Nobs,m,mmax,epsilon);

    for (i=0;i!=4;i++)
      for (j=0;j!=4;j++)
      {
        sigma[i][j] = 0.0;
        for (k=0;k!=K;k++)
          for (l=mmax+k;l!=Nobs;l++)
            sigma[i][j] += 4.0*ohm[k]*(A[i][l]*A[j][l-k]+A[i][l-k]*A[j][l])/(double)(2*(n-k));
      }

    d[0] = 1.0/C[1];
    d[1] = -C[0]/(C[1]*C[1]);
    d[2] = -1.0/C[3];
    d[3] = C[2]/(C[3]*C[3]);

    S2=0.0;
    for (i=0;i!=4;i++)
      for (j=0;j!=4;j++)
         S2 += d[i]*sigma[i][j]*d[j];

    HJ_TVAL = Q*sqrt(n)/sqrt(S2);
 
    HJ_Pval = 0.5 - .5*erf(HJ_TVAL/sqrt(2.0));
 



  fil=fopen(outfile,"w");
  fprintf(fil, "SampleN, Epsilon, HJstat, P-value\n");
  fprintf(fil, "%i,%f,%f,%f\n",Nobs, epsilon, HJ_TVAL,HJ_Pval);
  fclose(fil);

  return(0);
}

